var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ParsingError } from "@xieyuheng/x-data.js";
import fs from "node:fs";
import { createMod, modResolve } from "../mod/index.js";
import { parseStmts } from "../syntax/index.js";
export function load(url, loadedMods) {
    return __awaiter(this, void 0, void 0, function* () {
        const found = loadedMods.get(url.href);
        if (found !== undefined) {
            return found.mod;
        }
        const text = yield fs.promises.readFile(url.pathname, "utf8");
        try {
            const mod = createMod({ url, loadedMods });
            mod.stmts = parseStmts(text);
            loadedMods.set(url.href, { mod, text });
            for (const stmt of mod.stmts) {
                if (stmt.kind === "Import") {
                    const importedUrl = modResolve(mod, stmt.path);
                    yield load(importedUrl, loadedMods);
                }
            }
            return mod;
        }
        catch (error) {
            if (error instanceof ParsingError) {
                throw new Error(error.report(text));
            }
            throw error;
        }
    });
}
//# sourceMappingURL=load.js.map